@@ -9,15 +9,6 @@ XML::Filter::XML_Directory_2::Base - base class for creating XML::Directory to s
  package XML::Filter::XML_Directory_2Foo;
  use base qw (XML::Filter::XML_Directory_2::Base);
 
- sub start_element {
-   my $self = shift;
-   my $data = shift;
-
-   $self->on_enter_start_element($data) || return 0;
-
-   # do stuff here...
- }
-
 =head1 DESCRIPTION
 
 Base class for creating XML::Directory to something else SAX filters.
@@ -31,11 +22,10 @@ use strict;
 
 use Carp;
 use Exporter;
-use MIME::Types;
 use Digest::MD5 qw (md5_hex);
-use XML::Filter::XML_Directory_Pruner '1.1';
+use XML::Filter::XML_Directory_Pruner '1.2';
 
-$XML::Filter::XML_Directory_2::Base::VERSION   = '1.0';
+$XML::Filter::XML_Directory_2::Base::VERSION   = '1.4.1';
 @XML::Filter::XML_Directory_2::Base::ISA       = qw ( XML::Filter::XML_Directory_Pruner );
 @XML::Filter::XML_Directory_2::Base::EXPORT    = qw ();
 @XML::Filter::XML_Directory_2::Base::EXPORT_OK = qw ();
@@ -46,7 +36,7 @@ $XML::Filter::XML_Directory_2::Base::VERSION   = '1.0';
 
 This is a simple helper method designed to save typing. 
 
-Value arguments are 
+Value arguments are
 
 =over
 
@@ -87,6 +77,25 @@ sub attributes {
 
 =head1 OBJECT METHODS
 
+=head2 $pkg->exclude_root($bool)
+
+By default, XML::Directory will include the directory you pass to the I<XML::Directory(::SAX)::parse_dir> method. 
+
+You can use this method to instruct your filter to only include the contents of the root directory and not the directory itself.
+
+=cut
+
+sub exclude_root {
+  my $self = shift;
+  my $bool = shift;
+
+  if (defined($bool)) {
+    $self->{__PACKAGE__.'__includeroot'} = ($bool) ? 0 : 1;
+  }
+
+  return $self->{__PACKAGE__.'__includeroot'};
+}
+
 =head2 $pkg->start_level()
 
 Read-only.
@@ -109,6 +118,27 @@ sub cwd {
   return $self->{__PACKAGE__.'__cwd'};
 }
 
+=head2 $pkg->current_directory()
+
+Short-cut (ahem) for $pkg->cwd()
+
+=cut
+
+sub current_directory {
+  return $_[0]->cwd();
+}
+
+=head2 $pkg->current_location()
+
+Returns the current location relative to the directory root
+
+=cut
+
+sub current_location {
+  my $self = shift;
+  return $self->{__PACKAGE__.'__loc'};
+}
+
 =head2 $pkg->set_handlers(\%args)
 
 Define one or more valid SAX2 thingies to be called when your package encounters a specific event. Thingies are like any other SAX2 thingy with a few requirements :
@@ -121,10 +151,6 @@ Must inherit from XML::SAX::Base.
 
 =item *
 
-It's handler must be the same one passed to your class.
-
-=item *
-
 It must define a I<parse_uri> method.
 
 =back
@@ -179,11 +205,6 @@ sub set_handlers {
   foreach ($self->handler_events()) {
     next if (! $args->{$_});
 
-    if (! UNIVERSAL::isa($args->{$_},"XML::SAX::Base")) {
-      carp "Handler must be derived from XML::SAX::Base";
-      next;
-    }
-
     if (! UNIVERSAL::can($args->{$_},"parse_uri")) {
       carp "Handler must define a 'parse_uri' method.\n";
       next;
@@ -263,6 +284,10 @@ Returns an MD5 hash of the path, relative to the root, for the current file
 
 sub generate_id {
   my $self = shift;
+  print STDERR "GENERATE ID FOR ".$self->{__PACKAGE__.'__loc'}."\n";
+  my $id = "ID".&md5_hex($self->{__PACKAGE__.'__loc'});
+  print STDERR "ID IS $id\n";
+  return $id;
   return "ID".&md5_hex($self->{__PACKAGE__.'__loc'});
 }
 
@@ -306,44 +331,6 @@ sub make_link {
   return $link;
 }
 
-=head2 $pkg->mtype($file)
-
-Return the media type, as defined by the I<MIME::Types> package, associated with I<$file>.
-
-=cut
-
-sub mtype {
-  my $self  = shift;
-  my $fname = shift;
-
-  #
-
-  $fname =~ /^(.*)\.([^\.]+)$/;
-  if (! $2) { return undef; }
-
-  if (exists($self->{__PACKAGE__.'__typeof'}{$2})) {
-    return $self->{__PACKAGE__.'__typeof'}{$2};
-  }
-
-  $self->{__PACKAGE__.'__mtypes'} ||= MIME::Types->new()
-    || return undef;
-
-
-  #
-
-  my $mime  = $self->{__PACKAGE__.'__mtypes'}->mimeTypeOf($2);
-
-  if (! $mime) {
-    $self->{__PACKAGE__.'__typeof'}{$2} = undef;
-    return $self->{__PACKAGE__.'__typeof'}{$2};
-  }
-
-  #
-
-  $self->{__PACKAGE__.'__typeof'}{$2} = $mime->mediaType();
-  return $self->{__PACKAGE__.'__typeof'}{$2};
-}
-
 =head2 $pkg->on_enter_start_element(\%data)
 
 This method should be called as the first action in your class' I<start_element> method. It will perform a number of helper actions, like keeping track of the current node level and the absolute path of the current document.
@@ -367,11 +354,44 @@ sub on_enter_start_element {
       $self->{__PACKAGE__.'__head'} = 1;
   }
 
-  if ((! $self->{__PACKAGE__.'__start'}) && ($data->{Name} eq "directory")) {
-    $self->{__PACKAGE__.'__start'} = $self->current_level();
-    return 1;
+  if ($data->{Name} =~ /^(directory|file)$/) {
+    $self->{__PACKAGE__.'__'.$1} ++;
+#    map { print " "; } (0..$self->{__PACKAGE__.'__'.$1});
+#    print $self->{__PACKAGE__.'__'.$1} ." ($1) $data->{Attributes}->{'{}name'}->{Value} ".__PACKAGE__."\n";
   }
 
+  #
+
+  if ((! $self->{__PACKAGE__.'__start'}) && ($data->{Name} =~ /^(file|directory)$/)) {
+
+    if (! exists($self->{__PACKAGE__.'__includeroot'})) {
+      $self->{__PACKAGE__.'__start'} = $self->current_level();
+      return 1;
+    }
+
+    else {
+
+      if ((! $self->{__PACKAGE__.'__includeroot'}) &&
+	  (($self->{__PACKAGE__.'__file'} == 1) || ($self->{__PACKAGE__.'__directory'} == 2))) {
+
+	$self->{__PACKAGE__.'__start'} = $self->current_level();
+	$self->grow_cwd($data);
+
+	$self->compare($data);
+
+	if (! $self->skip_level()) {
+	  return 1;
+	}
+
+	$self->prune_cwd($data);
+	return 0;
+      }
+    }
+
+  }
+
+  #
+
   if (! $self->{__PACKAGE__.'__start'}) {
     return 0;
   }
@@ -416,6 +436,10 @@ sub on_exit_end_element {
     $self->prune_cwd($data);
   }
 
+  if ($data->{Name} =~ /^(directory|file)$/) {
+    $self->{__PACKAGE__.'__'.$1} --;
+  }
+
   $self->SUPER::on_exit_end_element($data);
   return 1;
 }
@@ -477,11 +501,11 @@ sub prune_cwd {
 
 =head1 VERSION
 
-1.0
+1.4.1
 
 =head1 DATE
 
-July 02, 2002
+July 07, 2002
 
 =head1 AUTHOR
 
